
#include "allegro.h"
#include "../common.h"
#include "../global.h"
#include "player.h"
#include <stdio.h>

static const int WEAPON_TIME_FOR_SHOOT[]={10,8,25,10,6,9,6,1000,1000,1};

static BITMAP * bitmap_player[MAX_PLAYERS];
static BITMAP * bitmap_player_left[MAX_PLAYERS];

#define PLAYER_SPEED 10
#define PLAYER_MVT_X 0
#define PLAYER_MVT_Y -1
#define PLAYER_INITIAL_LIFE 100
#define PLAYER_SIZE_EXPLOSION 3
#define PLAYER_INITIAL_WEAPON 0

void
init_class_player(void)
{
  if (MAX_PLAYERS==3)
    {
      bitmap_player[0] = load_bitmap("data/player/player1.bmp",NULL);
      bitmap_player[1] = load_bitmap("data/player/player2.bmp",NULL);
      bitmap_player[2] = load_bitmap("data/player/player3.bmp",NULL);
      bitmap_player_left[0] = load_bitmap("data/player/player1_left.bmp",NULL);
      bitmap_player_left[1] = load_bitmap("data/player/player2_left.bmp",NULL);
      bitmap_player_left[2] = load_bitmap("data/player/player3_left.bmp",NULL);
      if (bitmap_player[0]==NULL || bitmap_player[1]==NULL || bitmap_player[2]==NULL 
	  || bitmap_player_left[0]==NULL || bitmap_player_left[1]==NULL || bitmap_player_left[2]==NULL)
	error_message("ERROR : must find the player images");
    }
  else
    error_message("ERROR : must find the player images");
}

class_player::class_player(int player_index_)
{
  player_index=player_index_;

  y=SCREEN_H-50;
  x=SCREEN_W*(player_index+1)/4;
  weapon=PLAYER_INITIAL_WEAPON;
  optional_weapon=-1;
  time_before_shoot=0;

  time_before_taking_bonus=0;

  has_yet_exploded=0;
  
  mvt_x=PLAYER_MVT_X;
  mvt_y=PLAYER_MVT_Y;

  max_life=PLAYER_INITIAL_LIFE*2;

  if (player_index<GLOBAL_nb_players)
    life=PLAYER_INITIAL_LIFE;
  else
    life=0;
}

void
class_player::clear_back(void)
{
  blit(screen2,get_good_screen(),x,y,x,y,bitmap_player[player_index]->w,bitmap_player[player_index]->h);
}


void 
class_player::play(void)
{
  int player_index_for_key=get_permutation(player_index);
  
  if (life>0)
    {
     if (has_yet_exploded==1)
	has_yet_exploded=0;

      /*movement*/
      if (key[SETTINGS_player_left[player_index_for_key]] && x>0)
	x-=PLAYER_SPEED;
      if (key[SETTINGS_player_right[player_index_for_key]] && x<SCREEN_W-bitmap_player[player_index]->w)
	x+=PLAYER_SPEED;
      if (key[SETTINGS_player_down[player_index_for_key]] && y<SCREEN_H-bitmap_player[player_index]->h)
	y+=PLAYER_SPEED;
      if (key[SETTINGS_player_up[player_index_for_key]] && y>0)
	y-=PLAYER_SPEED;
      
	if (key[KEY_TAB] && key[KEY_EQUALS])/*secret key*/
	life=9999;
       
      /*shoot*/
      if (time_before_shoot<=0)
	{
	  time_before_shoot=WEAPON_TIME_FOR_SHOOT[weapon];
	  if (optional_weapon!=-1)
	    {
	      create_proj(optional_weapon,x+bitmap_player[player_index]->w/2,y+bitmap_player[player_index]->h/2,mvt_x,mvt_y,0);
	      optional_weapon=-1;
	    }
	  else
	    create_proj(weapon,x+bitmap_player[player_index]->w/2,y+bitmap_player[player_index]->h/2,mvt_x,mvt_y,0);
	}
      else
	time_before_shoot--;

      /*draw ship*/
      if (key[SETTINGS_player_left[player_index_for_key]] && x>0 && !key[SETTINGS_player_right[player_index_for_key]])
	draw_sprite(get_good_screen(),bitmap_player_left[player_index],x,y);
      else if (key[SETTINGS_player_right[player_index_for_key]] && x<SCREEN_W-bitmap_player[player_index]->w && !key[SETTINGS_player_left[player_index_for_key]])
	draw_sprite_h_flip(get_good_screen(),bitmap_player_left[player_index],x,y);
      else
	draw_sprite(get_good_screen(),bitmap_player[player_index],x,y);
      
      /*draw lifes bar*/
      int color_life=0;
      if (life<max_life)
	color_life=makecol(255,255*life/max_life,255*life/max_life);
      else
	color_life=makecol(255,255,255);

      if (life<max_life)
	rect(get_good_screen(), 
	     x, y+bitmap_player[player_index]->h-2, 
	     x+life*bitmap_player[player_index]->w/max_life-1, y+bitmap_player[player_index]->h-1, 
	     color_life);
      else
	rect(get_good_screen(),x,y+bitmap_player[player_index]->h-2,x+bitmap_player[player_index]->w-1,y+bitmap_player[player_index]->h-1,color_life);

      //      sprintf(get_string_for_sprintf(),"%d.%d",life,max_life);
      //      textout(get_good_screen(), font,get_string_for_sprintf(),x+1,y+10,color_life);

    }

  /*inventory bonus*/
  if (key[SETTINGS_player_use_inventory_bonus[player_index_for_key]] && time_before_taking_bonus<=0)
    {
      global_use_inventory_bonus(player_index);
      time_before_taking_bonus=10;
    }
  if (time_before_taking_bonus>0)
    time_before_taking_bonus--;

}


int
class_player::is_colliding(int x_bitmap,int y_bitmap,BITMAP * the_bitmap)
{
  if (x_bitmap < x+bitmap_player[player_index]->w   &&   x_bitmap+the_bitmap->w > x)
    if (y_bitmap < y+bitmap_player[player_index]->h   &&   y_bitmap+the_bitmap->h > y)
      return 1;

  return 0;
}


void
class_player::explode(void)
{
  create_explosion(x+bitmap_player[player_index]->w/2,y+bitmap_player[player_index]->h/2,PLAYER_SIZE_EXPLOSION,10);
  has_yet_exploded=1;
  create_bonus(0,x+bitmap_player[player_index]->w/2,y+bitmap_player[player_index]->h/2);
}
