
#include "allegro.h"
#include "../common.h"
#include "../global.h"
#include "ship.h"

#include "math.h"

#define NB_TYPES_SHIPS 8

static const int SHIP_SPEED[]={7,7,3,2,4,5,2,2};
static const int SHIP_WEAPON[]={7,8,3,1,7,9,4,6};
static const int SHIP_WEAPON2[]={-1,-1,9,8,-1,-1,-1,-1};
static const int SHIP_LIFE[]={1,5,30,30,50,60,90,120};
static const int SHIP_INITIAL_POSITION[]={0,0,0,0,2,0,0,0};
static const float INITIAL_SHIP_MVT_X[]={0,0,0.2,0,0,0,0,0};
static const float INITIAL_SHIP_MVT_Y[]={1,1,0.6,1,-1,1,1,1};
static const int SHIP_TIME_FOR_SHOOT[]={40,30,60,20,40,10,50,50};
static const int SHIP_TIME_FOR_SHOOT2[]={1,1,100,30,1,1,1,1};
static const int SHIP_SIZE_EXPLOSION[]={1,1,2,2,2,2,2,3};
static const int SHIP_EXPLOSION_DAMAGES[]={1,3,20,20,20,40,50,60};
static const int SHIP_TOTAL_MVT_MIN[]={250,300,200,800,300,250,250,250};
static const int SHIP_TOTAL_MVT_MAX[]={600,400,350,810,400,500,300,300};
static const int SHIP_TIME_STOP[]={60,40,50,0,60,0,100,120};
static const float AFTER_STOP_SHIP_MVT_X[]={0,0,0.4,0,0,0,0,0};
static const float AFTER_STOP_SHIP_MVT_Y[]={1,1,0.2,1,-1,1,1,1};

static BITMAP * bitmap_ship[NB_TYPES_SHIPS];

void 
init_class_ship(void)
{
  bitmap_ship[0] = load_bitmap("data/ship/ship0.bmp",NULL);
  bitmap_ship[1] = load_bitmap("data/ship/ship1.bmp",NULL);
  bitmap_ship[2] = load_bitmap("data/ship/ship2.bmp",NULL);
  bitmap_ship[3] = load_bitmap("data/ship/ship3.bmp",NULL);
  bitmap_ship[4] = load_bitmap("data/ship/ship4.bmp",NULL);
  bitmap_ship[5] = load_bitmap("data/ship/ship5.bmp",NULL);
  bitmap_ship[6] = load_bitmap("data/ship/ship6.bmp",NULL);
  bitmap_ship[7] = load_bitmap("data/ship/ship7.bmp",NULL);
}


class_ship::class_ship(int type_of_ship_)
{
  if (type_of_ship_>=NB_TYPES_SHIPS)
    error_message("ERROR : ship type unknown");
  
  type_of_ship=type_of_ship_;
  
  life=SHIP_LIFE[type_of_ship];
  mvt_x=INITIAL_SHIP_MVT_X[type_of_ship];
  mvt_y=INITIAL_SHIP_MVT_Y[type_of_ship];
  
  total_mvt = random_number(SHIP_TOTAL_MVT_MAX[type_of_ship]-SHIP_TOTAL_MVT_MIN[type_of_ship])+SHIP_TOTAL_MVT_MIN[type_of_ship];
  
  time_stop=SHIP_TIME_STOP[type_of_ship];
  
  after_stop_mvt_x=AFTER_STOP_SHIP_MVT_X[type_of_ship];
  after_stop_mvt_y=AFTER_STOP_SHIP_MVT_Y[type_of_ship];

  weapon=SHIP_WEAPON[type_of_ship];
  weapon2=SHIP_WEAPON2[type_of_ship];
  speed=SHIP_SPEED[type_of_ship];
  time_before_shoot=random_number(SHIP_TIME_FOR_SHOOT[type_of_ship]);
  time_before_shoot2=random_number(SHIP_TIME_FOR_SHOOT2[type_of_ship]);

  if (SHIP_INITIAL_POSITION[type_of_ship]==0)
    {
      x=random_number(SCREEN_W-bitmap_ship[type_of_ship]->w);
      y=random_number(300)-300-bitmap_ship[type_of_ship]->h;
    }
  else if (SHIP_INITIAL_POSITION[type_of_ship]==2)
    {
      x=random_number(SCREEN_W-bitmap_ship[type_of_ship]->w);
      y=random_number(300)+SCREEN_H;
    }
  
  initial_x=x;
  initial_y=y;
}

void
class_ship::clear_back(void)
{
  blit(screen2,get_good_screen(),x,y,x,y,bitmap_ship[type_of_ship]->w,bitmap_ship[type_of_ship]->h);
}

void 
class_ship::play(void)
{

  if (total_mvt>0)
    {
      x=x+(int)(mvt_x*speed);
      y=y+(int)(mvt_y*speed);
      
      total_mvt-=(int)sqrt( (mvt_x*speed)*(mvt_x*speed) + (mvt_y*speed)*(mvt_y*speed) );
      
    }
    else
    {
      time_stop--;
      if (time_stop<0)
	{
	  x=x+(int)(after_stop_mvt_x*speed);
	  y=y+(int)(after_stop_mvt_y*speed);
      }
    }
     
  if (time_before_shoot<=0)
    {
      time_before_shoot=SHIP_TIME_FOR_SHOOT[type_of_ship];
      create_proj(weapon,x+bitmap_ship[type_of_ship]->w/2,y+bitmap_ship[type_of_ship]->h/2,mvt_x,mvt_y,1);

      if (type_of_ship>=6)
	create_ship(0);
    }
  else
    time_before_shoot--;

  
  if (weapon2>-1)
    if (time_before_shoot2<=0)
      {
	time_before_shoot2=SHIP_TIME_FOR_SHOOT2[type_of_ship];
	create_proj(weapon2,x+bitmap_ship[type_of_ship]->w/2,y+bitmap_ship[type_of_ship]->h/2,mvt_x,mvt_y,1);
      }
    else
	time_before_shoot2--;
  

  if (x+bitmap_ship[type_of_ship]->w<-50 && initial_x+bitmap_ship[type_of_ship]->w>0) life=0;
  if (y+bitmap_ship[type_of_ship]->h<-50 && initial_y+bitmap_ship[type_of_ship]->h>0) life=0;
  if (x>SCREEN_W+50 && initial_x<SCREEN_W) life=0;
  if (y>SCREEN_H+50 && initial_y<SCREEN_H) life=0;

  if (x+bitmap_ship[type_of_ship]->w<-400) life=0;
  if (y+bitmap_ship[type_of_ship]->h<-400) life=0;
  if (x>SCREEN_W+400) life=0;
  if (y>SCREEN_H+400) life=0;
  
  draw_sprite(get_good_screen(),bitmap_ship[type_of_ship],x,y);
}


int
class_ship::is_colliding(int x_bitmap,int y_bitmap,BITMAP * the_bitmap)
{
  if (x_bitmap < x+bitmap_ship[type_of_ship]->w   &&   x_bitmap+the_bitmap->w > x)
    if (y_bitmap < y+bitmap_ship[type_of_ship]->h   &&   y_bitmap+the_bitmap->h > y)
      return 1;

  return 0;
}

void
class_ship::explode(void)
{
  create_explosion(x+bitmap_ship[type_of_ship]->w/2,y+bitmap_ship[type_of_ship]->h/2,SHIP_SIZE_EXPLOSION[type_of_ship],SHIP_EXPLOSION_DAMAGES[type_of_ship]);
  if (random_number(4)==1)
    create_bonus(0,x+bitmap_ship[type_of_ship]->w/2,y+bitmap_ship[type_of_ship]->h/2);
  if (random_number(15)==1)
    create_bonus(1,x+bitmap_ship[type_of_ship]->w/2,y+bitmap_ship[type_of_ship]->h/2);
}
