
#include "allegro.h"

#include "classes/menubox.h"
#include "classes/player.h"
#include "classes/ship.h"
#include "classes/proj.h"
#include "classes/explosion.h"
#include "classes/bonus.h"
#include "classes/other.h"

#include "common.h"

#include "global.h"

#include "inventory.h"

#include <math.h>
#include <unistd.h>

#define MAX_SHIPS 500
#define MAX_PROJS 2000
#define MAX_EXPLOSIONS 1500
#define MAX_BONUS 500
#define MAX_OTHERS 1000

class_ship * ship[MAX_SHIPS];
class_player * player[MAX_PLAYERS];
class_proj * proj[MAX_PROJS];
class_explosion * explosion[MAX_EXPLOSIONS];
class_bonus * bonus[MAX_BONUS];
class_other * other[MAX_OTHERS];

static int permutation_var=0;

void
init_all(void)
{
  init_all_commons();

  for (int i=0;i<MAX_SHIPS;i++)
    ship[i]=NULL;
  for (int i=0;i<MAX_PLAYERS;i++)
    player[i]=NULL;
  for (int i=0;i<MAX_PROJS;i++)
    proj[i]=NULL;
  for (int i=0;i<MAX_EXPLOSIONS;i++)
    explosion[i]=NULL;
  for (int i=0;i<MAX_BONUS;i++)
    bonus[i]=NULL;
  for (int i=0;i<MAX_OTHERS;i++)
    other[i]=NULL;

  init_class_ship();
  init_class_player();
  init_class_menubox();
  init_class_proj();
  init_class_explosion();
  init_class_bonus();
  init_class_other();
  inventory_init();
}

void
open_menubox(int x,int y,int size_x,int size_y,char * title)
{
  class_menubox * menubox = new class_menubox(x,y,size_x,size_y,title);
  delete menubox;
}

void
create_player(void)
{
  int i=0;

  while(i<MAX_PLAYERS && player[i]!=NULL)
    i++;

  if (i<MAX_PLAYERS)
    player[i] = new class_player(i);
  else
    error_message("ERROR : too much players already created");
}

void 
players_play(void)
{
  for (int i=0;i<MAX_PLAYERS;i++)
    {
      player[i]->play();
      if (player[i]->life<=0 && !player[i]->has_yet_exploded)
	{
	  player[i]->life=0;
	  player[i]->clear_back();
	  player[i]->explode();
	}
    }
}


void 
create_ship(int type_of_ship)
{
  int i=0;

  while(i<MAX_SHIPS && ship[i]!=NULL)
    i++;

  if (i<MAX_SHIPS)
    ship[i] = new class_ship(type_of_ship);
  else
    error_message("ERROR : too much ships already created");
  
}


void 
ships_play(void)
{
  for (int i=0;i<MAX_SHIPS;i++)
    if (ship[i]!=NULL)
      {
	ship[i]->play();
	if (ship[i]->life<=0)
	  {
	    ship[i]->clear_back();
	    ship[i]->explode();
	    delete ship[i];
	    ship[i]=NULL;
	  }
      }
}


void
create_proj(int type_of_proj,int x,int y,float mvt_x,float mvt_y,int is_baddy)
{
  int i=0;

  while(i<MAX_PROJS && proj[i]!=NULL)
    i++;
    
  if (i<MAX_PROJS)
    proj[i] = new class_proj(type_of_proj,x,y,(float)mvt_x,(float)mvt_y,is_baddy);
  else
    error_message("ERROR : too much projectiles already created");
  
}

void
projs_play(void)
{
  for (int i=0;i<MAX_PROJS;i++)
    if (proj[i]!=NULL)
      {
	proj[i]->play();
	if (proj[i]->life<=0)
	  {
	    proj[i]->clear_back();
	    proj[i]->explode();
	    delete proj[i];
	    proj[i]=NULL;
	  }
      }
}


int 
is_there_a_ship(void)
{
  for (int i=0;i<MAX_SHIPS;i++)
    if (ship[i]!=NULL)
      return 1;

  return 0;
}



void 
sprites_clear_background(void)
{ 
  inventory_clear();
  for (int i=0;i<MAX_PLAYERS;i++)
    player[i]->clear_back();
  for (int i=0;i<MAX_SHIPS;i++)
    if (ship[i]!=NULL)
	ship[i]->clear_back();
  for (int i=0;i<MAX_PROJS;i++)
    if (proj[i]!=NULL)
	proj[i]->clear_back();
  for (int i=0;i<MAX_EXPLOSIONS;i++)
    if (explosion[i]!=NULL)
	explosion[i]->clear_back();
  for (int i=0;i<MAX_OTHERS;i++)
    if (other[i]!=NULL)
	other[i]->clear_back();

  
  for (int i=0;i<MAX_BONUS;i++)
    if (bonus[i]!=NULL)
	bonus[i]->clear_back();
}

void
cause_collision_proj(void)
{
  for (int j=0;j<MAX_PROJS;j++)
    if (proj[j]!=NULL)
      {
	int x=proj[j]->x;
	int y=proj[j]->y;
	BITMAP * image=proj[j]->get_bitmap_of_proj();
	int binary_value=0;
	if (proj[j]->is_baddy)//if projectile lauched by ship (baddy)
	  {
	    for (int i=0;i<MAX_PLAYERS;i++)//test collision player
		if (player[i]->life > 0)
		  {
		  binary_value=player[i]->is_colliding(x,y,image);
		  if (binary_value) 
		    break;
		  }

	    if (SETTINGS_projs_collide_projs)
	      for (int i=0;i<MAX_PROJS;i++)//test collision projectile of player
		{
		  if (proj[i]!=NULL)
		    if (!proj[i]->is_baddy)
		      {
			binary_value=proj[i]->is_colliding(x,y,image);
			if (binary_value)
			  break;
		      }
		}
	    
	  }
	else//if projectile lauched by player
	  {

	    for (int i=0;i<MAX_SHIPS;i++)//test collision ship (baddy)
	      {
		if (ship[i]!=NULL)
		  {
		    binary_value=ship[i]->is_colliding(x,y,image);
		    if (binary_value)
			break;
		  }
	      }

	    if (SETTINGS_projs_collide_projs)
	      for (int i=0;i<MAX_PROJS;i++)//test collision projectile of ship
		{
		  if (proj[i]!=NULL)
		    if (proj[i]->is_baddy)
		      {
			binary_value=proj[i]->is_colliding(x,y,image);
			if (binary_value)
			  break;
		      }
		}
	    
	  }
	
	if (binary_value)
	  proj[j]->life=0;
      }
}


void
create_explosion(int x,int y,int size,int damages)
{
  int i=0;

  while(i<MAX_EXPLOSIONS && explosion[i]!=NULL)
    i++;
    
  if (i<MAX_EXPLOSIONS)
    explosion[i] = new class_explosion(x,y,size,damages);
  else
    error_message("ERROR : too much explosions already created");
  
}

void 
explosions_play(void)
{
  for (int i=0;i<MAX_EXPLOSIONS;i++)
    if (explosion[i]!=NULL)
      {
	explosion[i]->play();
	if (explosion[i]->life<=0)
	  {
	    explosion[i]->clear_back();
	    delete explosion[i];
	    explosion[i]=NULL;
	  }
      }
}

void
verify_collision_explosion(int x,int y, BITMAP * image,int damages)
{
  
  int binary_value=0;
  
  for (int i=0;i<MAX_PLAYERS;i++)//test collision player
    if (player[i]->life > 0)
      {
	binary_value=player[i]->is_colliding(x,y,image);
	if (binary_value) 
	  player[i]->life-=damages;
      }

  /*  
  for (int i=0;i<MAX_PROJS;i++)//test collision projectile
    {
      if (proj[i]!=NULL)
	binary_value=proj[i]->is_colliding(x,y,image);
      if (binary_value)
	proj[i]->explode();
    }
  */
  for (int i=0;i<MAX_SHIPS;i++)//test collision ship (baddy)
    {
      if (ship[i]!=NULL)
	{
	  binary_value=ship[i]->is_colliding(x,y,image);
	  if (binary_value && ship[i]->life>0)
	    ship[i]->life-=damages;
	}
    }
  
}

int
are_all_players_dead(void)
{
  for (int i=0;i<MAX_PLAYERS;i++)
    if (player[i]->life>0)
      return 0;
  
  return 1;
}

int 
get_a_cible(int is_proj_baddy)
{
  if (is_proj_baddy)
    {
      if (are_all_players_dead())
      	return -1;
      else
	{
	  int cible;
	  do
	    cible = random_number(MAX_PLAYERS);
	  while(player[cible]->life <= 0);
	  
	  return cible;
	}
    }
  else
    {
      if (is_there_a_ship())
	{
	  for (int i=0;i<MAX_SHIPS;i++)
	    if (ship[i]!=NULL)
	      return i;
	}
      else
	return -1;
    }
  return -1;
}


double
get_the_angle_to_cible(int x_proj,int y_proj,int index_cible,int is_proj_baddy)
{
  int dx;
  int dy;
  if (is_proj_baddy)
    { 
      if (index_cible==-1 || player[index_cible]->life<=0)
      	return 3.1415/2;

      dx = player[index_cible]->x - x_proj;
      dy = player[index_cible]->y - y_proj;
    }
  else
    {
      if (index_cible==-1 || ship[index_cible]==NULL)
	return 3*3.1415/2;
      
      dx = ship[index_cible]->x - x_proj;
      dy = ship[index_cible]->y - y_proj;
    }


  double hypo = sqrt(dx*dx+dy*dy);
  double cosinus = abs(dx)/hypo;
  double sinus = abs(dy)/hypo;
  if (dx<0) cosinus=-cosinus;
  if (dy<0) sinus=-sinus;

  if (sinus>0)
    return acos(cosinus);
  else
    return -acos(cosinus);

}





void 
create_bonus(int type_of_bonus,int x,int y)
{
  int i=0;

  while(i<MAX_BONUS && bonus[i]!=NULL)
    i++;

  if (i<MAX_BONUS)
    bonus[i] = new class_bonus(type_of_bonus,x,y);
  else
    error_message("ERROR : too much bonus already created");
  
}


void 
bonus_play(void)
{
  for (int i=0;i<MAX_SHIPS;i++)
    if (bonus[i]!=NULL)
      {
	bonus[i]->play();
	if (bonus[i]->life<=0)
	  {
	    bonus[i]->clear_back();
	    delete bonus[i];
	    bonus[i]=NULL;
	  }
      }

  inventory_show();
}


int
verify_collision_bonus(int x,int y, BITMAP * image,int type_of_bonus)
{
  int binary_value=0;

  for (int i=0;i<MAX_PLAYERS;i++)//test collision player
    if (player[i]->life > 0)
      {
	binary_value=player[i]->is_colliding(x,y,image);
	if (binary_value) 
	  {
	    if (inventory_add(type_of_bonus,image))
	      return 1;
	    else
	      return 0;
	  }
      }

  return 0;
}


void 
global_use_inventory_bonus(int index_player)
{
  int which_bonus=inventory_use();

  switch(which_bonus)
    {
    case -1:break;

    case 0:
      player[index_player]->life+=40;
      if (player[index_player]->life>player[index_player]->max_life)
	player[index_player]->life=player[index_player]->max_life;
      break;

    case 1:
      player[index_player]->optional_weapon=10;
      break;
      
    default:error_message("ERROR : unknown type of bonus");break;
    }

}




void 
create_other(int type_of_other)
{
  int i=0;

  while(i<MAX_OTHERS && other[i]!=NULL)
    i++;

  if (i<MAX_OTHERS)
    other[i] = new class_other(type_of_other);
  else
    error_message("ERROR : too much other already created");
  
}


void 
others_play(void)
{
  for (int i=0;i<MAX_SHIPS;i++)
    if (other[i]!=NULL)
      if (other[i]->type_of_other==0)
	{
	  other[i]->play();
	  if (other[i]->life<=0)
	    {
	      other[i]->clear_back();
	      delete other[i];
	      other[i]=NULL;
	    }
	}
  
  for (int i=0;i<MAX_SHIPS;i++)
    if (other[i]!=NULL)
      if (other[i]->type_of_other>0)
	{
	  other[i]->play();
	  if (other[i]->life<=0)
	    {
	      other[i]->clear_back();
	      delete other[i];
	      other[i]=NULL;
	    }
	}

}


int 
get_permutation(int player_index)
{
  if (SETTINGS_permutation_mode)
    {
      if (permutation_var+player_index>=MAX_PLAYERS)
	return permutation_var+player_index-MAX_PLAYERS;
      else
        return permutation_var+player_index;
    }
  else
    return player_index;

/*
  if (player[player_index]->life > 0)
    {
      int good;
      do 
	{
	  if (permutation_var+player_index>MAX_PLAYERS)
	    good = permutation_var+player_index-MAX_PLAYERS;
	  else
	    good = permutation_var+player_index;
	  player_index++;
	}
      while(player[good]->life>0);

      return good;
    }
  else
    return player_index;
 */
}


void 
permute(void)
{
  if (are_all_players_dead())
    return;
  
  permutation_var++;
  if (permutation_var>=MAX_PLAYERS)
    permutation_var=0;
}




void 
delete_all_game(void)
{

  for (int i=0;i<MAX_SHIPS;i++)
    if (ship[i]!=NULL)
      {
	delete ship[i];
	ship[i]=NULL;
      }
  for (int i=0;i<MAX_PLAYERS;i++)
    if (player[i]!=NULL)
      {
	delete player[i];
	player[i]=NULL;
      }
  for (int i=0;i<MAX_PROJS;i++)
    if (proj[i]!=NULL)
      {
	delete proj[i];
	proj[i]=NULL;
      }
  for (int i=0;i<MAX_EXPLOSIONS;i++)
    if (explosion[i]!=NULL)
    {
      delete explosion[i];
      explosion[i]=NULL;
    }
  for (int i=0;i<MAX_BONUS;i++)
    if (bonus[i]!=NULL)
    {
      delete bonus[i];
      bonus[i]=NULL;
    }
  for (int i=0;i<MAX_OTHERS;i++)
    if (other[i]!=NULL)
    {
      delete other[i];
      other[i]=NULL;
    }

  inventory_init();
}

void
set_good_weapon_players(int level_index)
{
  int index_weapon=0;

    switch(level_index)
      {
      case 1:index_weapon=0;break;
      case 2:index_weapon=1;break;
      case 3:index_weapon=2;break;
      case 4:index_weapon=3;break;
      case 5:index_weapon=4;break;
      case 6:index_weapon=5;break;
      case 7:index_weapon=6;break;
      default:error_message("ERROR : level weapon unknown for set_good_weapon_players()");break;
      }


  for (int i=0;i<MAX_PLAYERS;i++)
    player[i]->weapon=index_weapon;
}


void 
cinematic_end(void)
{
  for (int i=0;i<50;i++)
    {
      sprites_clear_background();
      
      create_explosion(random_number(SCREEN_W-64),0,2,0);
      
      others_play();
      players_play();
      ships_play();
      explosions_play();
      bonus_play();

      refresh();
      
      wait_a_moment(20);
      /*      rest_callback(20,(*callback_exit));*/
    }

  rectfill(screen,0,0,SCREEN_W,SCREEN_H,0);
  
  draw_menu();

  while(!is_auto_quit_menu())
    open_menubox(SCREEN_W/2-80,SCREEN_H/2,1,1,"Fin du jeu");
  
}
